#!/bin/sh
# Run this to generate all the initial makefiles, etc.
# 

srcdir=`dirname $0`
test -z "$srcdir" && srcdir=.

PKG_NAME="DConnect Daemon"

##### Tests part:
##########################################

(test -f $srcdir/configure.in \
	&& test -d $srcdir/src) || {
	echo -n "**Error**: Directory "\`$srcdir\'" does not look like the"
	echo " top-level rc-scripts directory"
	exit 1
}

DIE=0

(autoconf --version) < /dev/null > /dev/null 2>&1 || {
	echo
	echo "**Error**: You must have \`autoconf' installed to compile $PKG_NAME."
	echo "Download the appropriate package for your distribution,"
	echo "or get the source tarball at ftp://ftp.gnu.org/pub/gnu/"
	DIE=1
}

(grep "^AM_PROG_LIBTOOL" $srcdir/configure.in >/dev/null) && {
(libtoolize --version) < /dev/null > /dev/null 2>&1 || {
	echo
	echo "**Error**: You must have \`libtool' installed to compile $PKG_NAME."
	echo "Get ftp://ftp.gnu.org/pub/gnu/libtool-1.2d.tar.gz"
	echo "(or a newer version if it is available)"
	DIE=1
}
}

grep "^AM_GNU_GETTEXT" $srcdir/configure.in >/dev/null && {
grep "sed.*POTFILES" $srcdir/configure.in >/dev/null || \
(gettextize --version) < /dev/null > /dev/null 2>&1 || {
	echo
	echo "**Error**: You must have \`gettext' installed to compile $PKG_NAME."
	echo "Get ftp://alpha.gnu.org/gnu/gettext-0.10.35.tar.gz"
	echo "(or a newer version if it is available)"
	DIE=1
}
}

(automake --version) < /dev/null > /dev/null 2>&1 || {
echo
echo "**Error**: You must have \`automake' installed to compile $PKG_NAME."
echo "Get ftp://ftp.gnu.org/pub/gnu/automake-1.3.tar.gz"
echo "(or a newer version if it is available)"
DIE=1
NO_AUTOMAKE=yes
}


# if no automake, don't bother testing for aclocal
test -n "$NO_AUTOMAKE" || (aclocal --version) < /dev/null > /dev/null 2>&1 || {
echo
echo "**Error**: Missing \`aclocal'.  The version of \`automake'"
echo "installed doesn't appear recent enough."
echo "Get ftp://ftp.gnu.org/pub/gnu/automake-1.3.tar.gz"
echo "(or a newer version if it is available)"
DIE=1
}

if test "$DIE" -eq 1; then
	exit 1
fi

if test -z "$*"; then
	echo "**Warning**: I am going to run \`configure' with no arguments."
	echo "If you wish to pass any to it, please specify them on the"
	echo \`$0\'" command line."
	echo
fi

case $CC in
	xlc )
	   am_opt=--include-deps;;
esac

##### ac/am/libtool part regeneration
#####################################
if grep "^AM_PROG_LIBTOOL" configure.in >/dev/null; then
	echo "Running libtoolize..."
	libtoolize --copy --force
fi

if grep "^AM_GNU_GETTEXT" configure.in >/dev/null; then
	if grep "sed.*POTFILES" configure.in >/dev/null; then
		: do nothing -- we still have an old unmodified configure.in
	else
		echo "Creating $dr/aclocal.m4 ..."
		echo "Running gettextize...  Ignore non-fatal messages."
		echo "no" | gettextize --force --copy
		echo "Making $dr/aclocal.m4 writable ..."
		test -r $dr/aclocal.m4 && chmod u+w $dr/aclocal.m4
	fi
fi

echo "Running aclocal $aclocalinclude ..."
aclocal $aclocalinclude

echo "Running autoconf ..."
autoconf

if grep "^AM_CONFIG_HEADER" configure.in >/dev/null; then
	echo "Running autoheader..."
	autoheader
fi

echo "Running automake --add-missing --gnu -c $am_opt ..."
automake --add-missing --gnu -c $am_opt


##### configure part
######################################

# temporary empty:
conf_flags=""

if test x$NOCONFIGURE = x; then
	echo Running $srcdir/configure $conf_flags "$@" ...
	$srcdir/configure $conf_flags "$@" \
	&& echo Now type \`make\' to compile $PKG_NAME
else
	echo Skipping configure process.
fi

