/* Borrowed from GNU Wget *
 * Modified by rawsock    */

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include "main.h"

const char *commands[]={"listening_port",
                        "hubname",
                        "user_limit",
                        "allow_broken_key",
                        "ping_timeout",
                        "register_timeout",
                        "nickname_pattern",
                        "min_share",
                        ""};

/* 
 * Copy the string formed by two pointers to a new location
 */
char *strdupdelim(const char *beg,const char *end)
{
 char *res=(char *)malloc(end-beg+1);
 memcpy(res,beg,end-beg);
 res[end-beg]=0;
 return res;
}

/* Syntax: <sp>* command <sp>* = <sp>* value <newline>
   Return values:	 1 - success
		         0 - failure
			-1 - empty   */
int parse_line(const char *line,char **com,char **val)
{
 const char *p=line;
 const char *orig_comptr,*end;
 char *new_comptr;
 while(*p&&isspace(*p)) ++p;
 if (!*p||*p=='#') return -1;
 for (orig_comptr=p;isalpha(*p)||*p=='_'||*p=='-';p++);
 if (!isspace(*p)&&(*p!='=')) return 0;
 *com=(char *)malloc(p-orig_comptr+1);
 for (new_comptr=*com;orig_comptr<p;orig_comptr++)
 { //if (*orig_comptr=='_'||*orig_comptr=='-') continue;
   *new_comptr++ = *orig_comptr;
 }
 *new_comptr=0;
 for (;isspace(*p);p++);
 if (*p!='=') { free (*com); return 0; }
 for (++p;isspace(*p);p++);
 end=line+strlen(line)-1;
 while(end>p&&isspace(*end)) --end;
 *val=strdupdelim(p,end+1);
 return 1;
}

/*
 * Read a line from FP and return the pointer to allocated storage.
 */
char *read_whole_line(FILE *fp)
{
 int length=0;
 int bufsize=82;
 char *line=(char *)malloc(bufsize);
 while(fgets(line+length,bufsize-length,fp))
 {
  length+=strlen(line+length);
  if (length==0) continue;
  if (line[length-1]=='\n') break;
  bufsize<<=1;
  line=realloc(line,bufsize);
 }
 if (length==0||ferror(fp)) { free (line); return NULL; }
 if (length+1<bufsize) line=realloc(line,length+1);
 return line;
}

/*
 * Parse config file
 */
void parse_config(struct config *conf)
{
 char *line;
 int ln;
 FILE *fp;
 
 /*
  * We look for config in location taken from configure
  * If not found - take default values
  */
 snprintf(conf->file, sizeof(CONFIGDIR)+sizeof(CONFIG_FILE), "%s/%s", CONFIGDIR, CONFIG_FILE);
 fp=fopen(conf->file,"rb");
 if (!fp) {
	 puts("Can't open config file!!");
	 /* logging via syslog */
	 openlog ("dcd", LOG_PID | LOG_CONS, LOG_DAEMON);
	 syslog(LOG_WARNING, "WARNING: Can't read config file - using default values");
	 perror("fopen");
	 return;
 }
 ln=1;
 while(line=read_whole_line(fp))
 {
  char *com,*val;
  int status;
  status=parse_line(line,&com,&val);
  free(line);
  if (status==1)
  {
   status=0;
   while(*commands[status])
   { if (!strcasecmp(com,commands[status])) break;
     else status++;
   }
   switch(status)
   {
    // listening_port
    case 0: if ((status=atoi(val))==0)
            { fprintf(stderr,"Error in %s at line %d.\n",conf->file,ln);
		    /* logging via syslog */
		    openlog ("dcd", LOG_PID | LOG_CONS, LOG_DAEMON);
		    syslog(LOG_WARNING, "WARNING: Error in %s at line %d.\n",conf->file,ln);
		    break; }
            conf->listen_port=status;
            break;
    // hubname
    case 1: if (conf->hubname) free(conf->hubname);
            conf->hubname=(char *)malloc(strlen(val)+1);
            strcpy(conf->hubname,val);
            break;
    // user_limit
    case 2: conf->userlimit=atoi(val);
            break;
    // allow_broken_key
    case 3: conf->allow_broken_key=atoi(val);
            break;
    // ping_timeout
    case 4: conf->ping_timeout=atoi(val);
            break;
    // register_timeout
    case 5: conf->register_timeout=atoi(val);
            break;
    // nickname_pattern
    case 6: if (conf->nick_pattern) free(conf->nick_pattern);
            conf->nick_pattern=(char *)malloc(strlen(val)+1);
            strcpy(conf->nick_pattern,val);
            break;
    // min_share
    case 7: conf->minshare=atoi(val);
            break;
    // action when everything fails:
    default: fprintf(stderr,"Error in %s at line %d.\n",conf->file,ln);
	     /* logging via syslog */
	     openlog ("dcd", LOG_PID | LOG_CONS, LOG_DAEMON);
	     syslog(LOG_WARNING, "WARNING: Error in %s at line %d.\n",conf->file,ln);
   }
   free(com);
   free(val);
  }
  else if (status==0) fprintf(stderr,"Error in %s at line %d.\n",conf->file,ln);
  ++ln;
 }
 fclose(fp);
}
