// $Id: list.h,v 1.3 2002/05/18 20:03:53 ventox Exp $

/* ---------------------------------------------------------------- 
 *  DConnect Daemon for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dconnect.ds.pg.gda.pl
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- */

/* 
 * $Log: list.h,v $
 * Revision 1.3  2002/05/18 20:03:53  ventox
 * - some errors corrected
 *
 * Revision 1.1  2002/04/27 18:21:10  ventox
 * Initial revision
 *
 */

#if !defined( __vXhz__LIST_H__INCLUDED__ )
#define __vXhz__LIST_H__INCLUDED__

// initial list threshold
#define INITIAL_VLIST_THRESHOLD   ((_int32b)0x0000000AL)

typedef struct 
  {
    _int32b _threshold;     // threshold
    _int32b _size;          // actual list capacity
    void * *_items;         // items holder
    _int32b _position;      // current insert position
    _int32b _seek;          // actual seek position
  } 
  _vlist, * _vlist_ptr;

/* creation & distraction */  

#define _vlist_CREATE( _vl ) \
  _vlist_create( &_vl, INITIAL_VLIST_THRESHOLD )

// creates and sets up the given list
_status_t _vlist_create( _vlist ** vl, _int32b threshold );

#define _vlist_DESTROY( _vl ) \
  _vlist_destroy( &_vl, NULL );
   
// destroys and cleans up the given list
_status_t _vlist_destroy( _vlist ** vl, void (* clean_up)( void * ) );
 
/* vlist state */
 
// returns TRUE if there's no elements on the list
#if defined( _DEBUG )
  BOOL _vlist_is_empty( _vlist * vl );
#else /* _DEBUG */
  #define _vlist_is_empty( _vl ) \
    (BOOL)( ((_vlist *)(_vl))->_position == 0 )
#endif /* !_DEBUG */

// returns FALSE if element 'item' isn't on the 'vl' list
#define _vlist_has_item( _vl, _i ) \
  (BOOL)( _vlist_index_of( (_vlist *)_vl, (void *)_i ) != -1 )

// gives total number of elements in the list  
#if defined( _DEBUG )
  _int32b _vlist_get_number_of_items( _vlist * vl );
#else /* _DEBUG */
  #define _vlist_get_number_of_items( _vl ) \
    ((_vlist *)(_vl))->_position
#endif /* !_DEBUG */

/* simple read-only access */

// resets seek position
#define _vlist_reset_seek_position( _vl ) \
  { \
    ASSERT( vl != NULL ); \
    ((_vlist *)(_vl))->_seek = 0; \
  } \

// gets seek position
#if defined( _DEBUG )
  _int32b _vlist_get_seek_position( _vlist * vl );
#else /* _DEBUG */
  #define _vlist_get_seek_position( _vl ) \
    ((_vlist *)(_vl))->_seek
#endif /* !_DEBUG */
  
// sets seek position
#define _vlist_set_seek_position( _vl, _value ) \
  { \
    ASSERT( _vl != NULL ); \
    ((_vlist *)(_vl))->_seek = (_value); \
  } \

// gets element stored at '_seek' position on the list  
#define _vlist_get_at_seek_position( _vl ) \
    _vlist_get_item_at( (_vlist *)_vl, ((_vlist *)_vl)->_seek )

// sets new element at '_seek' position on the list
#define _vlist_set_at_seek_position( _vl, _i ) \
  { \
    ASSERT( _vl != NULL ); \
    _vlist_set_item_at( _vl, _i, ((_vlist *)_vl)->_seek ); \
  } \
  
// gets first element from the list
void * _vlist_first_item( _vlist * vl );

// gets next element from the list
void * _vlist_next( _vlist * vl );

// gets last element from the list
void * _vlist_last_item( _vlist * vl );

// gets previous element from the list
void * _vlist_previous( _vlist * vl );

/* full access to the elements */

// sets element at given position into the list
void _vlist_set_item_at( _vlist * vl, _vXhz_CONST void * item , _int32b index );

// gets element at given position from the list
#if defined( _DEBUG )
  void * _vlist_get_item_at( _vlist * vl, _int32b index );
#else /* _DEBUG */
  #define _vlist_get_item_at( _vl, _i ) \
    ((_vlist *)(_vl))->_items[(_i)]
#endif /* !_DEBUG */

// returns the position of the item on the list
_int32b _vlist_index_of( _vlist * vl, _vXhz_CONST void * item );

// returns the position of the item on the list (reverse search)
_int32b _vlist_last_index_of( _vlist *vl, _vXhz_CONST void * item );

// inserts element at given position from the list
void * _vlist_insert_at( _vlist * vl, _vXhz_CONST void * item, _int32b index );

// inserts element into the list
_int32b _vlist_insert( _vlist * vl, _vXhz_CONST void * item );

// removes element at given position from the list
void * _vlist_remove_at( _vlist * vl, _int32b index );

// removes element from the the list
_int32b _vlist_remove( _vlist * vl, _vXhz_CONST void * item );

// removes all elements from the list
void _vlist_remove_all( _vlist * vl );

// deletes all elements from the list
void _vlist_delete_all( _vlist * vl, void (* clean_up)( void * ) );
    
/* miscellaneous */

// swaps two elements from the list
void _vlist_swap( _vlist * vl, _int32b i1, _int32b i2 );
    
#define _vlist_FOREACH( _vl, _fp ) \
  _vlist_foreach( _vl, _fp, NULL );

// call 'test_proc' with each element form the list
_status_t _vlist_foreach( _vlist * vl, _status_t (* test)( void *, void * ), void * );

#endif /* __vXhz__LIST_H__INCLUDED___ */
