static char rcs_id[] =
  "$Id: main.c,v 1.42 2002/05/31 15:17:31 blues Exp $";

/* ---------------------------------------------------------------- 
 *  DirectConnect HUB for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- */

/* include for PCH file	*/
#include "pch.h"

/*
 * $Log: main.c,v $
 * Revision 1.42  2002/05/31 15:17:31  blues
 * - few words in README and added web site address in welcome message.
 *
 * Revision 1.41  2002/05/30 15:30:04  blues
 * - some things should be send to syslog
 *
 * Revision 1.40  2002/05/28 18:40:19  sebo
 * Simple config file parser and minimum share requirements.
 * Please view the changes and test it.
 *
 * Revision 1.39  2002/05/28 10:34:59  blues
 * - nicer comments
 *
 * Revision 1.38  2002/05/18 12:54:09  blues
 * - some cosmetics in comments
 *
 * Revision 1.37  2002/05/14 09:59:51  sebo
 * - empty users bugfix
 * - registration timeout introduced (default = 30sec.)
 * - a listen() backlog issue corrected
 *
 * Revision 1.36  2002/05/08 11:00:42  sebo
 * Some update
 *
 * Revision 1.35  2002/05/07 08:42:27  sebo
 * Problem with libwrap hopfuly solved.
 *
 * Revision 1.34  2002/05/07 08:24:20  sebo
 * Small fix.
 *
 * Revision 1.33  2002/05/06 22:03:56  sebo
 * Ignoring too long lines instead of shutting down hub. Not tested 'cause I'm going to bed ;-)
 *
 * Revision 1.32  2002/05/06 18:54:31  sebo
 * $MultiConnectToMe "support"
 *
 * Revision 1.31  2002/04/28 12:36:19  sebo
 * Daemon mode now works (I hope) and is the default. `-d' debug mode introduced.
 *
 * Revision 1.30  2002/04/28 12:27:26  blues
 * - move this logging to main function (do not show on every thread create)
 *
 * Revision 1.29  2002/04/28 12:21:51  blues
 * - Started logging to syslog
 *
 * Revision 1.28  2002/04/27 18:21:10  ventox
 * Initial revision
 *
 * 
 */

/* include local header files	*/
#include "main.h"

#include "conf.h"
#include "misc.h"
#include "net.h"

/* users table	*/
struct userrec *user[MAXUSERS];

/* configuration data	*/
struct config conf;
int NUSERS;
int linerd=1;			// line-read-and-ready flag
/* assume command length (from '$' to '|') doesn't exceed BUF_SIZE	*/
char line[BUF_SIZE];
pthread_t listen_th;
fd_set sockfdset;
struct sockaddr_in local;
int listen_sock;

int allow_severity=LOG_INFO;
int deny_severity=LOG_WARNING;
#ifdef HAVE___PROGNAME
extern char *__progname;
#else
char *__progname;
#endif

struct request_info req;

/* allocate and init new user	*/
void adduser( int * sock, char * remote ) {
 char lock[128];
 struct userrec *tmp;
 struct timeval tv;
 tmp=(struct userrec *)malloc(sizeof(struct userrec));
 bzero(tmp,sizeof(struct userrec));
 tmp->sock=dup(*sock);
 if (setsockopts(&(tmp->sock))<0) perror("setsockopt");
	/* 
	 * we use select() anyway, but let's care for blocking send()
	 */
 if (fcntl(tmp->sock,F_SETFL,O_NONBLOCK)<0) perror("fcntl");
 lockgen(lock);
 sendf_to(tmp,"$Lock %s",lock);
 sendf_to(tmp,"$HubName %s",conf.hubname);
	/* 
	 * client complains that it couldn't find $Lock when msgs are sent before it
	 */
 publicmsg(NULL,tmp,"This Hub is running DConnect Daemon for Linux. http://www.dc.ds.pg.gda.pl");
 if (denied(&(tmp->sock))) { disconnect(tmp,"You're not allowed to connect. Good bye."); return; }
 if (banned(remote)) { disconnect(tmp,"Your IP is banned!"); return; }
 if (NUSERS>=conf.userlimit)
    { disconnect(tmp,"Sorry, this Hub is currently Full!"); return; }
 strcpy(tmp->ip,remote);
 gettimeofday(&tv,NULL);
 tmp->ping=tv.tv_sec;
 tmp->flag=1;
 user[NUSERS++]=tmp;
 FD_SET(tmp->sock,&sockfdset);
}

/*
 * flush user's data
 */
void deluser(int id,char *reason) {
 struct userrec *tmp;
 if ((id<0)||(id>=MAXUSERS)||(!user[id])) return;
 tmp=user[id];
 user[id]=user[--NUSERS];
 user[NUSERS]=NULL;
 FD_CLR(tmp->sock,&sockfdset);
 if (tmp->state>=2) sendf_mcast("$Quit %s",tmp->nick);
 disconnect(tmp,reason);
}

/*
 * signal handler and program termination
 */
void finish(int sig)
{
 int n;
 switch(sig)
 {
  /* What for is that signal? - blues */
  case SIGPIPE:	return;
  /* Reload config file when you get SIGHUP */
  case SIGHUP:	read_config(&conf);
		/* logging via syslog */
		openlog ("dcd", LOG_PID | LOG_CONS, LOG_DAEMON);
		syslog(LOG_NOTICE, "Reloaded configuration");
		return;
  /* Shutdown HUB on proper signals. 
   * What singals should be here?? - blues */
  default:	pthread_cancel(listen_th);
		pthread_join(listen_th,NULL);
		for (n=0;n<NUSERS;n++) disconnect(user[n],"Hub shutdown!");
		/* logging via syslog */
		openlog ("dcd", LOG_PID | LOG_CONS, LOG_DAEMON);
		syslog(LOG_NOTICE, "Shutting down, DConnect Daemon");
  		exit(sig);
 }
}

/*
 * point 'pos' to the beginning of a new line in 'buf'
 */
static int buf2line(int *pos,char *buf) {
 int x=0;
 char tmp[BUF_SIZE];
 if (!buf[*pos]) return 0;
 if (linerd) *line=0;
 while((buf[*pos]!='|')&&buf[*pos]&&(x<BUF_SIZE-1)) tmp[x++]=buf[(*pos)++];
 tmp[x]=0;
 if (strlen(line)+x>BUF_SIZE-1)
    { puts("Line too long - ignored."); *line=0; return linerd=0; }
 strcat(line,tmp);
 if (!buf[*pos]) linerd=0; else { (*pos)++; linerd=1; }
 return linerd;
}

/*
 * proces requests remaining on a socket
 */
static void manage_sock( int id ) {
 
  const char *commands[]={"Key",
                         "Quit",
			 "ValidateNick",
                         "GetNickList",
			 "MyINFO",
			 "GetINFO",
			 "Kick",
			 "MyPass",
			 "Search",
			 "MultiSearch",
			 "To:",
			 "Version",
			 "ConnectToMe",
			 "MultiConnectToMe",
			 ""};
 const char *sep=" |";
 char buf[BUF_SIZE],tmp[513],*cmd,*par;
 struct timeval tv;
 int pos=0,count,n;
 if ((count=recv(user[id]->sock,buf,BUF_SIZE,0))<=0)
    { deluser(id,NULL); return; }
 buf[count]=0;
 if (user[id]->state==2) { gettimeofday(&tv,NULL); user[id]->ping=tv.tv_sec; }
 while(buf2line(&pos,buf))
 {
  if (*line) { printf("[%s] %s\n",*(user[id]->nick)?user[id]->nick:"Unknown",line); fflush(stdout); }
  strcpy(tmp,line);
  if (!(cmd=strtok(tmp,sep))) continue;
  par=strtok(NULL,sep);
  count=0;
  if (*cmd=='$')
     while(*commands[count]) { if (!strcasecmp(&cmd[1],commands[count])) break;
                               else count++; }
  else if (*cmd=='<') count=-1;
       else count=-2;
  if (((user[id]->state==0)&&(count!=0))||
      ((user[id]->state==1)&&(count!=2))||
      ((user[id]->state==2)&&(count==2))) { deluser(id,NULL); return; }
  switch(count)
  {
   // Msg
   case -1: sprintf(tmp,"<%s>",user[id]->nick);
            if (!strcasecmp(cmd,tmp)) send_mcast(line);
	    else puts("Msg: Fake nick not allowed.");
            continue;
   // Key
   case 0: if (!validkey(id,par)) return;
           if (user[id]->state==0) user[id]->state=1;
           continue;
   // Quit
   case 1: if (!par) continue;
           if (strcasecmp(par,user[id]->nick))
              { puts("Quit: Fake nick not allowed."); continue; }
           deluser(id,NULL);
           return;
   // ValidateNick
   case 2: if (!validnick(id,par)) return;
           user[id]->state=2;
           sendf_mcast("$Hello %s",user[id]->nick);
           privmsg(NULL,user[id],"Welcome to DConnect Deamon for Linux");
           continue;
   // GetNickList
   case 3: cmd=(char *)malloc(21*NUSERS+12);
           if (!cmd) { puts("GetNickList: Memory allocation error."); continue; }
           strcpy(cmd,"$NickList ");
           for(n=0;n<NUSERS;n++)
	      if (user[n]->state==2) { strcat(cmd,user[n]->nick); strcat(cmd,"$$"); }
	   send_to(user[id],cmd);
	   free(cmd);
           continue;
   // MyINFO
   case 4: if (fill_info(user[id],line))
           { if (user[id]->share<(conf.minshare*1024*1024))
	     { sprintf(tmp,"You must share at least %LuMB on this hub.",conf.minshare);
	       deluser(id,tmp);
	     }
	     else send_mcast(line);
	   }
           continue;
   // GetINFO
   case 5: if (!par) continue;
           if ((n=nick2id(par))<0)
	      { puts("GetINFO: No such user."); continue; }
	   sendf_to(user[id],"$MyINFO $ALL %s %s$ $%s%c$%s$%Lu$",user[n]->nick,user[n]->desc,user[n]->ctype,user[n]->flag,user[n]->email,user[n]->share);
           continue;
   // Kick
   case 6:
           continue;
   // MyPass
   case 7:
           continue;
   // Search
   case 8: if (!par) continue;
           strtok(par,":");
           if (!strcmp(user[id]->ip,par)) send_mcast(line);
	   else puts("Search: Incorrect IP.");
           continue;
   // MultiSearch
   case 9:
           continue;
   // To:
   case 10: if (!par) continue;
            if ((n=nick2id(par))<0) { puts("To: No such user."); continue; }
            if (!(par=strtok(NULL,sep))) continue;
            if (strcasecmp(par,"From:")) continue;
            if (!(par=strtok(NULL,sep))) continue;
            if (strcasecmp(user[id]->nick,par))
               { puts("From: Fake nick not allowed."); continue; }
            send_to(user[n],line);
            continue;
   // Version
   case 11: if (!par) continue;
            strcpy(user[id]->ver,par);
            continue;
   // ConnectToMe
   case 12: if (!par) continue;
            if ((n=nick2id(par))<0)
	       { puts("ConnectToMe: No such user."); continue; }
            if (!(par=strtok(NULL,":"))) continue;
	    if (strcmp(user[id]->ip,par))
	       { puts("ConnectToMe: Incorrect IP."); continue; }
	    send_to(user[n],line);
            continue;
   // MultiConnectToMe (?)
   case 13: if (!par) continue;
            if ((n=nick2id(par))<0)
	       { puts("MultiConnectToMe: No such user."); continue; }
            if (!(par=strtok(NULL,":"))) continue;
	    if (strcmp(user[id]->ip,par))
	       { puts("MultiConnectToMe: Incorrect IP."); continue; }
	    send_to(user[n],line);
            continue;
   default: printf("Unknown command \"%s\" - ignored.\n",cmd);
  }
 }
}

/* listen for new connections */
void * listen_thread( void * args ) 
{
 int new_sock,remote_size;
 struct sockaddr_in remote;
 char *ip;
 setuid(65534);
 setgid(65534);
 while(1)
 {
	  remote_size=sizeof(struct sockaddr_in);
	  if ( (new_sock = accept( listen_sock,(struct sockaddr *)&remote,&remote_size))<0 )
	  {
	     if (errno!=EINTR) 
	       { perror("accept"); finish(0); }
	     else 
	       continue;
	  }
	  printf("Connection from %s:%d\n",ip=inet_ntoa(remote.sin_addr),ntohs(remote.sin_port));
	  adduser(&new_sock,ip);
	  close(new_sock);
 }
 pthread_exit(0);
}

/* install the signal handler and initialize variables */
static void init_var( char *argv0 )
{
 signal(SIGPIPE,finish);
 signal(SIGHUP,finish);
 signal(SIGINT,finish);
 signal(SIGTERM,finish);
 FD_ZERO(&sockfdset);
 NUSERS=0;
 bzero(user,MAXUSERS*sizeof(struct userrec *));
 bzero(&conf,sizeof(struct config));
 conf.file=(char *)malloc(strlen(CONFIG_FILE)+1);
 strcpy(conf.file,CONFIG_FILE);

 /* set up default values */
 conf.listen_port=DEFAULT_PORT;
 conf.daemon=DAEMON;
 conf.hubname=(char *)malloc(strlen(HUBNAME)+1);
 strcpy(conf.hubname,HUBNAME);
 conf.userlimit=USERLIMIT;
 conf.allow_broken_key=ALLOW_BROKEN_KEY;
 conf.ping_timeout=PING_TIMEOUT;
 conf.register_timeout=REGISTER_TIMEOUT;
 conf.nick_pattern=(char *)malloc(strlen(NICK_PATTERN)+1);
 strcpy(conf.nick_pattern,NICK_PATTERN);
 /* end of default values */
 
 __progname=get_progname(argv0);
 randomize();
}

int main(int argc,char *argv[])
{
 int i,maxfd;
 struct timeval timeout;
 fd_set readfdset;
 /* init variables */
 init_var(argv[0]);
 
 /* parse command-line options */
 parse_options( &conf, argc, argv );
 
 /* read config file */
 read_config(&conf);
 
 maxfd=getdtablesize();
 
 /* logging via syslog */
 openlog ("dcd", LOG_PID | LOG_CONS, LOG_DAEMON);
 syslog(LOG_NOTICE, "Initializing, DConnect Daemon");
 
 /* setup and bind a listening socket */
 if ((listen_sock=socket(AF_INET,SOCK_STREAM,0))==-1)
    { perror("socket"); finish(0); }
 setsockopts(&listen_sock);
 local.sin_family=AF_INET;
 local.sin_port=htons(conf.listen_port);
 local.sin_addr.s_addr=INADDR_ANY;
 bzero(&(local.sin_zero),8);
 if (bind(listen_sock,(struct sockaddr *)&local,sizeof(struct sockaddr))<0)
    { perror("bind"); finish(0); }
 if (listen(listen_sock,5)<0)
    { perror("listen"); finish(0); }
 /* daemonize and set owner to nobody */
 if (conf.daemon) if (daemon(0,0)<0) perror("daemon");
 setuid(65534);
 setgid(65534);
 /* create listening thread */
 if (pthread_create( &listen_th, NULL, (void *)listen_thread, NULL)!=0)
    { perror("pthread_create"); finish(0); }
 /* main loop - finish only when error or killed with a signal */
 while(1)
 {
  /* 
   * we use 1ms timeout because we want select() to proces new users even when
   * there is no traffic on the sockets
   */
  timeout.tv_sec=0; timeout.tv_usec=1000;
  bcopy(&sockfdset,&readfdset,sizeof(readfdset));

   if (select(maxfd,&readfdset,NULL,NULL,&timeout)<0)
   {
     if (errno!=EINTR) 
       { perror("select"); finish(0); }
     else 
       continue;
   }
   
  /* this needs to be a reverse loop because of the method used in deluser() */
  for (i=NUSERS-1;i>=0;i--) 
  { 
    if (user[i]) {
      if (FD_ISSET(user[i]->sock,&readfdset))
        manage_sock(i);
    }
    if (user[i]) check_timeout(i);
  }
 }
 
 finish( 0 );
 return 0;
}
