// $Id: main.h,v 1.17 2002/05/28 18:40:19 sebo Exp $

/* ---------------------------------------------------------------- 
 *  DirectConnect HUB for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: ???
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- */

#if !defined( __vXhz__MAIN_H__INCLUDED__ )
#define __vXhz__MAIN_H__INCLUDED__ 

/*
 * $Log: main.h,v $
 * Revision 1.17  2002/05/28 18:40:19  sebo
 * Simple config file parser and minimum share requirements.
 * Please view the changes and test it.
 *
 * Revision 1.16  2002/05/14 09:59:51  sebo
 * - empty users bugfix
 * - registration timeout introduced (default = 30sec.)
 * - a listen() backlog issue corrected
 *
 * Revision 1.15  2002/05/06 22:03:56  sebo
 * Ignoring too long lines instead of shutting down hub. Not tested 'cause I'm going to bed ;-)
 *
 * Revision 1.14  2002/05/06 18:56:38  blues
 * - Use project name as default name
 *
 * Revision 1.13  2002/05/06 18:54:35  blues
 * - use 1000 users by default
 *
 * Revision 1.12  2002/04/28 12:36:19  sebo
 * Daemon mode now works (I hope) and is the default. `-d' debug mode introduced.
 *
 * Revision 1.11  2002/04/27 18:21:10  ventox
 * Initial revision
 *
 *
 */

void adduser( int * sock, char * remote );

void deluser( int, char * );
void finish( int );

/* define default values */

// buffer size - shouldn't be touched
#define BUF_SIZE 1024

// daemon's listening port number
#define DEFAULT_PORT 411

// 1 = detach from tty, 0 = keep connected; can be overriden with -d
#define DAEMON 1

// max number of simultanously connected clients (for static buffers)
#define MAXUSERS 1000

// limit for number of users (dc protocol level); <=MAXUSERS (!!!)
#define USERLIMIT 1000

// name of our hub, obviously
#define HUBNAME "DConnectDaemon"

// set to 1, if hub should allow clients to connect even when they reply
// with an invalid $Key string
#define ALLOW_BROKEN_KEY 0

// ping timeout duration in seconds; 0 = no timeout
#define PING_TIMEOUT 0

// registration time limit in seconds; 0 = no limit
#define REGISTER_TIMEOUT 30

// default path to config file
#define CONFIG_FILE "dchub.conf"

// required nick pattern
#define NICK_PATTERN "*"

/* end of default values */

// record with user's data
struct userrec
{
 int sock;
 char state;
 char nick[20];
 char ip[16];
 char ver[16];
 long int ping;
 unsigned long long int share;
 char flag;
 char ctype[256];
 char desc[256];
 char email[256];
};

// filled with config file data
struct config
{
 char *file;
 char daemon;
 unsigned short int listen_port;
 char *hubname;
 int userlimit;
 char allow_broken_key;
 char ping_timeout;
 char register_timeout;
 char *nick_pattern;
 unsigned long long int minshare;
 int minslots;
};

#endif /* __vXhz__MAIN_H__INCLUDED__ */
