static char rcs_id[] =
    "$Id: misc.c,v 1.25 2002/05/31 17:55:29 blues Exp $";

/* ---------------------------------------------------------------- 
 *  DirectConnect HUB for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: ???
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- */

// include for PCH file
#include "pch.h"

/* 
 * $Log: misc.c,v $
 * Revision 1.25  2002/05/31 17:55:29  blues
 * - fixed handling config, reverted my mistakes, fixed initscripts and specs
 *
 * Revision 1.24  2002/05/28 18:40:19  sebo
 * Simple config file parser and minimum share requirements.
 * Please view the changes and test it.
 *
 * Revision 1.23  2002/05/18 20:07:31  ventox
 * - type name 'status_t' changed to '_status_t'
 *
 * Revision 1.22  2002/05/14 09:59:51  sebo
 * - empty users bugfix
 * - registration timeout introduced (default = 30sec.)
 * - a listen() backlog issue corrected
 *
 * Revision 1.21  2002/05/08 11:00:43  sebo
 * Some update
 *
 * Revision 1.20  2002/05/07 08:42:27  sebo
 * Problem with libwrap hopfuly solved.
 *
 * Revision 1.19  2002/05/06 22:03:56  sebo
 * Ignoring too long lines instead of shutting down hub. Not tested 'cause I'm going to bed ;-)
 *
 * Revision 1.18  2002/04/28 12:36:19  sebo
 * Daemon mode now works (I hope) and is the default. `-d' debug mode introduced.
 *
 * Revision 1.17  2002/04/27 18:21:10  ventox
 * Initial revision
 *
 *
 */

// include local header files
#include "main.h"
#include "conf.h"
#include "match.h"

extern struct userrec *user[];
extern struct config conf;
extern int NUSERS;
extern struct request_info req;
extern char *__progname;

static char * g_module_name;

/* some helper functions */
char * _to_lower( char * _s )
  {
    char * _t = _s;
    ASSERT( _t );
    
    while ( *_t != '0' )
      *(_t++) = tolower( *_t );

    return _s;
  }

/* module */
  
_status_t _module__finalize( void )
  {
    if ( g_module_name != NULL )
      free( g_module_name );
    
    return OK;
  }

_status_t _module__initialize( char ** argv )
  {
    /* module name & location */
    char * _t
      = strrchr( *argv, _PATH );
    
    if ( _t != NULL ) 
      _t++; 
    else 
      _t = *argv;
        
    g_module_name = 
      (char *)malloc( strlen( _t ) * sizeof( char ) );    
    
    ASSERT( g_module_name );    
    
    _to_lower( 
      strcpy( g_module_name, _t ) 
    );
    
    /* options */
    return OK;
  }
  
_vXhz_CONST char * _module__get_name( void )
  { return (_vXhz_CONST char *)g_module_name; }
 
  // just show a brief usage
void show_usage( void )
  {
    printf( "\nDConnect Hub.\n" );
/* not working, so I use __progname for now - rawsock */
//    printf( "Usage:\t%s [options]\n\n", _module__get_name() );
    printf( "Usage:\t%s [options]\n\n", __progname );
    printf( "-h\t\t this help screen\n" );
    printf( "-c config_file\t use alternate config file\n" );
    printf( "-d\t\t debug mode\n\n" );
    exit( 0 );
  }

/*
 * parse command-line options
 */
void parse_options( struct config *conf, int argc, char * argv[] )
  {
    int i;

/* seg fault with this (?!) - rawsock */
//    _module__initialize( argv );
    
    for ( i = 1; i < argc; i++ )
   {
    if ( argv[i][0] != '-' ) 
      show_usage( );
  
    switch(argv[i][1])
    {
     case 'h': show_usage(); break;
     case 'c': if (argc<i+2) show_usage(); else i++;
               if (conf->file) free(conf->file);
               conf->file=(char *)malloc(strlen(argv[i])+1);
               strcpy(conf->file,argv[i]);
               break;
     case 'd': conf->daemon=0; break;
     default: printf("Unknown option: `%s'. Try `-h' for help.\n\n",argv[i]);
              exit(0);
  }
 }
}

/*
 * from openssh
 */
char *get_progname(char *argv0)
{
#ifdef HAVE___PROGNAME
 extern char *__progname;
 return __progname;
#else
 char *p;
 if (argv0==NULL) return "unknown";
 p=strrchr(argv0,'/');
 if (p==NULL) p=argv0; else p++;
 return p;
#endif
}

/*
 * feed a random numbers generator
 */
void randomize( void )
{
  unsigned int seed;

  FILE * rdev = fopen( "/dev/urandom", "r" );
  if ( rdev != NULL ) {
    seed = fgetc( rdev ); fclose( rdev );
  }
 
  seed += getpid() + time( NULL );
  srand( seed );
}

// lock generator: 'salt Pk=privkey'
void lockgen(char *lock)
{
  int salt_len, pk_len, n;
  
  salt_len = 50 + rand() % 50;
  pk_len = 5;
 
  for(n=0;n<salt_len;n++) lock[n]='%'+rand()%('z'-'%');
 lock[salt_len]=0;
 strcat(lock," Pk=");
 for(n=0;n<pk_len;n++) lock[salt_len+4+n]='%'+rand()%('z'-'%');
 lock[salt_len+4+pk_len]=0;
}

// check if a given nick is a valid one
int validnick(int id,char *nick)
{
 int n=0;
 char reason[256];
 const char *reserved[]={"Admin",
                         "Hub",
			 "Hub-Security",
			 "Vandel\\Debug",
			 "Client",
			 ""};
 if (!nick) { deluser(id,NULL); return 0; }
 if (!*nick) { deluser(id,NULL); return 0; }
 if (*(conf.nick_pattern))
    if (!match(conf.nick_pattern,nick))
    {
     sprintf(reason,"Your nick doesn't match the `%s' pattern.",conf.nick_pattern);
     deluser(id,reason);
     return 0;
    }
 while(*reserved[n]) if (!strcasecmp(nick,reserved[n++]))
                        { deluser(id,"This nick is reserved!"); return 0; }
 for(n=0;n<strlen(nick);n++)
    if ((nick[n]<33)||(nick[n]>126)||(nick[n]=='$'))
       { deluser(id,"Your nick is invalid!"); return 0; }
 for(n=0;n<NUSERS;n++)
    if (!strcmp(user[n]->nick,nick))
       { deluser(id,"Your nick is already in use!"); return 0; }
 strcpy(user[id]->nick,nick);
 return 1;
}

// check for a valid $Key sequence
int validkey(int id,char *key)
{
 if (conf.allow_broken_key) return 1;
 if (!key) { deluser(id,NULL); return 0; }
 /* checking here */
 return 1;
}

// fill user's record with data received in $MyINFO
int fill_info(struct userrec *usr,char *info)
{
 char *i,*tmp,*bak;
 bak=(char *)malloc(strlen(info)+1);
 strcpy(bak,info);
 tmp=strchr(bak,' '); if (!tmp) { free(bak); return 0; } tmp++;
 i=strchr(tmp,' '); if (!i) { free(bak); return 0; } *i=0;
 if (strcasecmp(tmp,"$ALL")) { puts("MyINFO: Unknown format."); free(bak); return 0; }
 tmp=i+1;
 i=strchr(tmp,' '); if (!i) { free(bak); return 0; } *i=0;
 if (strcasecmp(tmp,usr->nick)) { puts("MyINFO: Fake nick not allowed."); free(bak); return 0; }
 tmp=i+1;
 i=strchr(tmp,'$'); if (!i) { free(bak); return 0; } *i=0;
 if (strlen(tmp)>sizeof(usr->desc)-1) { puts("fill_info(): Buffer overflow!"); free(bak); return 0; }
 strcpy(usr->desc,tmp);
 tmp=i+1;
 i=strchr(tmp,'$'); if (!i) { free(bak); return 0; }
 tmp=i+1;
 i=strchr(tmp,'$'); if (!i) { free(bak); return 0; } *i=0;
 usr->flag=*(i-1); *(i-1)=0;
 if (strlen(tmp)>sizeof(usr->ctype)-1) { puts("fill_info(): Buffer overflow!"); free(bak); return 0; }
 strcpy(usr->ctype,tmp);
 tmp=i+1;
 i=strchr(tmp,'$'); if (!i) { free(bak); return 0; } *i=0;
 if (strlen(tmp)>sizeof(usr->email)-1) { puts("fill_info(): Buffer overflow!"); free(bak); return 0; }
 strcpy(usr->email,tmp);
 tmp=i+1;
 i=strchr(tmp,'$'); if (!i) { free(bak); return 0; } *i=0;
 sscanf(tmp,"%Lu",&(usr->share));
 free(bak);
 return 1;
}

// convert user's nick to table index
int nick2id(char *nick)
{
 int id;
 for(id=0;id<NUSERS;id++) if (!strcasecmp(user[id]->nick,nick)) return id;
 return -1;
}

// check clients' times
void check_timeout(int id)
{
 struct timeval tv;
 gettimeofday(&tv,NULL);
 if (conf.ping_timeout)
 if (tv.tv_sec-user[id]->ping>conf.ping_timeout)
    { deluser(id,"Ping timeout."); return; }
 if (conf.register_timeout&&(user[id]->state!=2))
 if (tv.tv_sec-user[id]->ping>conf.register_timeout)
    { deluser(id,"Registration timeout."); return; }
}

// check if remote host is not allowed to connect
int denied(int *sock)
{
 request_init(&req,RQ_DAEMON,__progname,RQ_FILE,*sock,0);
 fromhost(&req);
 if (hosts_access(&req)) return 0;
 return 1;
}

// check for banned IPs
// read banned entries from config file
char banned(char *remote)
{
 return 0;
}

// read configuration data from file
void read_config(struct config *conf)
{
 unsigned short int oldport;
 oldport=conf->listen_port; // backup current listening port
 /* here read config file and fill conf struct */
 parse_config(conf);
 /* end of reading */
 /* we should restart the listening thread after listen_port has changed
    - not supported yet */
 //if (oldport!=conf->listen_port) restart_listen_thread();
}
